from coingecko.api.coins import Coins
from typing import Union

class ProCoins(Coins):
    def get_list_new(self) -> list:
        """🪙 Get the latest 200 coins (id & activated time) that recently listed on CoinGecko.com."""
        endpoint = 'coins/list/new'

        return self.get_data(endpoint)

    def get_top_gainers_losers(self, vs_currency: str, duration: str = "24h", top_coins: Union[int, str] = 1000) -> dict:
        """📈 get the top 30 coins with largest price gain and loss by a specific time duration"""
        endpoint = 'coins/top_gainers_losers'

        params = {
            'vs_currency': vs_currency,
            'duration': duration,
            'top_coins': top_coins
        }

        return self.get_data(endpoint, params=params)

    def get_circulating_supply_chart(self, id: str, days: Union[str, int], interval: str = None) -> dict:
        """📈 Get historical circulating supply of a coin, by number of days away from now."""
        endpoint = f'coins/{id}/circulating_supply_chart'

        params = {
            'days': days,
            'interval': interval
        }

        return self.get_data(endpoint, params=params)

    def get_circulating_supply_chart_range(self, id: str, from_timestamp: Union[str, int], to_timestamp: Union[str, int]) -> dict:
        """📈 Get historical circulating supply of a coin, within a range of timestamp."""
        endpoint = f'coins/{id}/circulating_supply_chart/range'

        params = {
            'from': from_timestamp,
            'to': to_timestamp
        }

        return self.get_data(endpoint, params=params)

    def get_total_supply_chart(self, id: str, days: Union[str, int], interval: str = None) -> dict:
        """📈 Get historical total supply of a coin, by number of days away from now."""
        endpoint = f'coins/{id}/total_supply_chart'

        params = {
            'days': days,
            'interval': interval
        }

        return self.get_data(endpoint, params=params)

    def get_total_supply_chart_range(self, id: str, from_timestamp: Union[str, int], to_timestamp: Union[str, int]) -> dict:
        """📈 Get historical total supply of a coin, within a range of timestamp."""
        endpoint = f'coins/{id}/total_supply_chart/range'

        params = {
            'from': from_timestamp,
            'to': to_timestamp
        }

        return self.get_data(endpoint, params=params)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class GlobalData(CoinGeckoAPI):
    def get(self) -> dict:
        """Get cryptocurrency global data"""
        endpoint = 'global'

        return self.get_data(endpoint)

    def get_defi(self) -> dict:
        """Get cryptocurrency global decentralized finance(defi) data"""
        endpoint = 'global/decentralized_finance_defi'

        return self.get_data(endpoint)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class Nfts(CoinGeckoAPI):
    def get_list(self, order: str = None, asset_platform_id: str = None, per_page: int = 100, page: int = 1) -> list:
        """List all supported NFT ids, paginated by 100 items per page, paginated to 100 items"""
        endpoint = 'nfts/list'

        params = {
            'order': order,
            'asset_platform_id': asset_platform_id,
            'per_page': per_page,
            'page': page
        }

        return self.get_data(endpoint, params=params)

    def get_id(self, id: str) -> dict:
        """Get currnet data (name, price_floor, volume_24h) for an NFT collection"""
        endpoint = f'nfts/{id}'

        return self.get_data(endpoint)

    def get_by_contract_address(self, asset_platform_id: str, contract_address: str) -> dict:
        """Get currnet data (name, price_floor, volume_24h) for an NFT collection"""
        endpoint = f'nfts/{asset_platform_id}/contract/{contract_address}'

        return self.get_data(endpoint)



from coingecko.demo_client import CoinGeckoDemoClient
from coingecko.pro_client import CoinGeckoProClient

from coingecko.api.error import GeckoAPIBadRequest, GeckoAPIUnauthorized, GeckoAPITooManyRequests, GeckoAPIInternalServerError, GeckoAPIServiceUnavailable, GeckoAPIException



from coingecko.api.base import CoinGeckoAPI

class TokenLists(CoinGeckoAPI):
    def get_all(self, asset_platform_id: str) -> dict:
        """🪙 Get full list of tokens of a blockchain network (asset platform) that is supported by Ethereum token list standard."""

        endpoint = f'token_lists/{asset_platform_id}/all.json'

        return self.get_data(endpoint)



from .ping import Ping
from .simple import Simple
from .coins import Coins
from .asset_platform import AssetPlatform
from .categories import Categories
from .exchanges import Exchanges
from .derivatives import Derivatives
from .nfts import Nfts
from .exchange_rates import ExchangeRates
from .global_data import GlobalData
from .search import Search
from .companies import Companies
import requests

class CoinGeckoClient:
    def __init__(self, api_key: str, pro: bool = False):
        self.api_key = api_key
        self.pro = pro
        self.session = requests.Session()
        self._ping = None
        self._simple = None
        self._coins = None
        self._asset_platform = None
        self._categories = None
        self._exchanges = None
        self._derivatives = None
        self._nfts = None
        self._exchange_rates = None
        self._global_data = None
        self._search = None
        self._companies = None

    @property
    def ping(self):
        if self._ping is None:
            self._ping = Ping(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._ping

    @property
    def simple(self):
        if self._simple is None:
            self._simple = Simple(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._simple

    @property
    def coins(self):
        if self._coins is None:
            self._coins = Coins(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._coins

    @property
    def asset_platform(self):
        if self._asset_platform is None:
            self._asset_platform = AssetPlatform(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._asset_platform

    @property
    def categories(self):
        if self._categories is None:
            self._categories = Categories(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._categories

    @property
    def exchanges(self):
        if self._exchanges is None:
            self._exchanges = Exchanges(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._exchanges

    @property
    def derivatives(self):
        if self._derivatives is None:
            self._derivatives = Derivatives(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._derivatives

    @property
    def nfts(self):
        if self._nfts is None:
            self._nfts = Nfts(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._nfts

    @property
    def exchange_rates(self):
        if self._exchange_rates is None:
            self._exchange_rates = ExchangeRates(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._exchange_rates

    @property
    def global_data(self):
        if self._global_data is None:
            self._global_data = GlobalData(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._global_data

    @property
    def search(self):
        if self._search is None:
            self._search = Search(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._search

    @property
    def companies(self):
        if self._companies is None:
            self._companies = Companies(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._companies



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args
from typing import List, Union

class Coins(CoinGeckoAPI):
    @process_function_args
    def get_list(self, include_platform: bool = False) -> list:
        """List all supported coins id, name and symbol (no pagination required)"""
        endpoint = 'coins/list'

        params = {
            'include_platform': include_platform
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_markets(self, vs_currency: str, ids: Union[str, List[str]] = None,
                        category: str = None, order: str = "market_cap_desc", per_page: int = 100,
                        page: int = 1, sparkline: bool = False, price_change_percentage: str = None,
                        locale: str = "en", precision: Union[str, int] = None) -> list:
        """List all supported coins price, market cap, volume, and market related data"""
        endpoint = 'coins/markets'

        params = {
            'vs_currency': vs_currency,
            'ids': ids,
            'category': category,
            'order': order,
            'per_page': per_page,
            'page': page,
            'sparkline': sparkline,
            'price_change_percentage': price_change_percentage,
            'locale': locale,
            'precision': precision,
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_id(self, id: str, localization: bool = True, tickers: bool = True,
                      include_exchange_logo: bool = False, market_data: bool = True,
                      community_data: bool = True, developer_data: bool = True,
                      description_data: bool = True, sparkline: bool = False) -> dict:
        """Get current data (name, price, market, ... including exchange tickers) for a coin"""
        endpoint = f'coins/{id}'

        params = {
            'localization': localization,
            'tickers': tickers,
            'include_exchange_logo': include_exchange_logo,
            'market_data': market_data,
            'community_data': community_data,
            'developer_data': developer_data,
            'description_data': description_data,
            'sparkline': sparkline
        }


        return self.get_data(endpoint, params=params)

    # Note: To handle pagination later
    @process_function_args
    def get_tickers(self, id: str, exchange_ids: Union[str, List[str]] = None,
                        include_exchange_logo: bool = False, page: int = None,
                        order: str = "trust_score_desc", depth: bool = False) -> dict:
        """Get coin tickers (paginated to 100 items)"""
        endpoint = f'coins/{id}/tickers'

        params = {
            'exchange_ids': exchange_ids,
            'include_exchange_logo': include_exchange_logo,
            'page': page,
            'order': order,
            'depth': depth
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_history(self, id: str, date: str, localization: bool = None) -> dict:
        """Get historical data (name, price, market, stats) at a given date for a coin"""
        endpoint = f'coins/{id}/history'

        params = {
            'date': date,
            'localization': localization
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_market_chart(self, id: str, vs_currency: str, days: Union[str, int],
                             interval: str = None, precision: Union[str, int] = None) -> dict:
        """Get historical market data include price, market cap, and 24h volume (granularity auto)"""
        endpoint = f'coins/{id}/market_chart'

        params = {
            'vs_currency': vs_currency,
            'days': days,
            'interval': interval,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_market_chart_range(self, id: str, vs_currency: str, from_timestamp: Union[str, int],
                                   to_timestamp: Union[str, int], interval: str = None, precision: Union[str, int] = None) -> dict:
        """Get historical market data include price, market cap, and 24h volume within a range of timestamp (granularity auto)"""
        endpoint = f'coins/{id}/market_chart/range'

        params = {
            'vs_currency': vs_currency,
            'from': from_timestamp,
            'to': to_timestamp,
            'interval': interval,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_ohlc(self, id: str, vs_currency: str, days: Union[str, int],
                     interval: str = None, precision: Union[str, int] = None) -> dict:
        """Get coin's OHLC (candlestick) data"""
        endpoint = f'coins/{id}/ohlc'

        params = {
            'vs_currency': vs_currency,
            'days': days,
            'interval': interval
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_coin_by_contract_address(self, id: str, contract_address: str) -> dict:
        """Get coin info from contract address"""
        endpoint = f'coins/{id}/contract/{contract_address}'

        return self.get_data(endpoint)

    @process_function_args
    def get_market_chart_by_contract_address(self, id: str, contract_address: str, vs_currency: str,
                                                 days: Union[str, int], interval: str = None,
                                                 precision: [str, int] = None) -> dict:
        """Get historical market data include price, market cap, and 24h volume (granularity auto) from a contract address"""
        endpoint = f'coins/{id}/contract/{contract_address}/market_chart'

        params = {
            'vs_currency': vs_currency,
            'days': days,
            'interval': interval,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_market_chart_range_by_contract_address(self, id: str, contract_address: str, vs_currency: str,
                                                       from_timestamp: Union[str, int], to_timestamp: Union[str, int],
                                                       interval: str = None,
                                                       precision: Union[str, int] = None) -> dict:
        """Get historical market data include price, market cap, and 24h volume within a range of timestamp (granularity auto) from a contract address"""
        endpoint = f'coins/{id}/contract/{contract_address}/market_chart/range'

        params = {
            'vs_currency': vs_currency,
            'from': from_timestamp,
            'to': to_timestamp,
            'interval': interval,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class Exchanges(CoinGeckoAPI):
    @process_function_args
    def get(self, per_page: int = None, page: str = None) -> list:
        """List all exchanges"""
        endpoint = 'exchanges'

        params = {
            'per_page': per_page,
            'page': page
        }

        return self.get_data(endpoint, params=params)

    def get_list(self) -> list:
        """List all supported markets id and name (no pagination required)"""
        endpoint = 'exchanges/list'

        return self.get_data(endpoint)

    def get_id(self, id: str) -> dict:
        """Get exchange volume in BTC and Top 100 Tickers only"""
        endpoint = f'exchanges/{id}'

        return self.get_data(endpoint)

    def get_tickers(self, id: str, coin_ids: str = None, include_exchange_logo: bool = False,
                    page: int = None, depth: bool = False, order: str = "trust_score_desc") -> dict:
        """Get exchange tickers (paginated, 100 tickers per page)"""
        endpoint = f'exchanges/{id}/tickers'

        params = {
            'coin_ids': coin_ids,
            'include_exchange_logo': include_exchange_logo,
            'page': page,
            'depth': depth,
            'order': order
        }

        return self.get_data(endpoint, params=params)

    def get_volume_chart(self, id: str, days: str) -> list:
        """Get 24 hour rolling trading volume data (in BTC) for a given exchange"""
        endpoint = f'exchanges/{id}/volume_chart'

        params = {
            'days': days
        }

        return self.get_data(endpoint, params=params)






from coingecko.api.global_data import GlobalData
from typing import Union


class ProGlobalData(GlobalData):
    def get_market_cap_chart(self, days: Union[str, int], vs_currency: str = 'usd') -> dict:
        """📈 Get historical global market cap and volume data, by number of days away from now."""
        endpoint = 'global/market_cap_chart'

        params = {
            'vs_currency': vs_currency,
            'days': days
        }

        return self.get_data(endpoint, params=params)



from coingecko.api.base import CoinGeckoAPI
from typing import List, Union
from coingecko.api.utils import process_function_args

class Simple(CoinGeckoAPI):
    @process_function_args
    def get_price(self, ids: Union[str, List[str]], vs_currencies: Union[str, List[str]],
                         include_market_cap: bool = False,
                         include_24hr_vol: bool = False, include_24hr_change: bool = False,
                         include_last_updated_at: bool = False, precision: Union[str, int] = None) -> dict:
        """Get current price of any cryptocurrency in any other currency that you need."""
        endpoint = 'simple/price'

        params = {
            'ids': ids,
            'vs_currencies': vs_currencies,
            'include_market_cap': include_market_cap,
            'include_24hr_vol': include_24hr_vol,
            'include_24hr_change': include_24hr_change,
            'include_last_updated_at': include_last_updated_at,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)

    @process_function_args
    def get_token_price(self, id: str, contract_addresses: Union[str, List[str]], vs_currencies: Union[str, List[str]],
                                  include_market_cap: bool = False,
                                  include_24hr_vol: bool = False, include_24hr_change: bool = False,
                                  include_last_updated_at: bool = False, precision: Union[str, int] = None) -> dict:
        """Get current price of tokens for a given platform in any other currency that you need."""
        endpoint = f'simple/token_price/{id}'

        params = {
            'contract_addresses': contract_addresses,
            'vs_currencies': vs_currencies,
            'include_market_cap': include_market_cap,
            'include_24hr_vol': include_24hr_vol,
            'include_24hr_change': include_24hr_change,
            'include_last_updated_at': include_last_updated_at,
            'precision': precision
        }

        return self.get_data(endpoint, params=params)


    def get_supported_vs_currencies(self) -> list:
        """Get list of supported_vs_currencies."""
        endpoint = 'simple/supported_vs_currencies'

        return self.get_data(endpoint)




from coingecko.api.nfts import Nfts
from typing import Union

class ProNfts(Nfts):
    def get_markets(self, asset_platform_id: str = None, order: str = "market_cap_usd_desc", per_page: int = 100, page: int = 1) -> list:
        """🖼️ Get the list of all supported NFT floor price, market cap, volume and market related data on CoinGecko"""
        endpoint = 'nfts/markets'

        params = {
            'asset_platform_id': asset_platform_id,
            'order': order,
            'per_page': per_page,
            'page': page
        }

        return self.get_data(endpoint, params=params)

    def get_market_chart(self, id: str, days: Union[str, int]) -> list:
        """📈 Get historical market data of a NFT collection, including floor price, market cap, and 24h volume, by number of days away from now."""

        endpoint = f'nfts/{id}/market_chart'

        params = {
            'days': days
        }

        return self.get_data(endpoint, params=params)

    def get_market_chart_by_contract_address(self, asset_platform_id: str, contract_address: str, days: Union[str, int]) -> dict:
        """📈  Get historical market data of a NFT collection using contract address, including floor price, market cap, and 24h volume, by number of days away from now."""

        endpoint = f'nfts/{asset_platform_id}/contract/{contract_address}/market_chart'

        params = {
            'days': days
        }

        return self.get_data(endpoint, params=params)

    def get_tickers(self, id: str) -> dict:
        """🖼️ Get the latest floor price and 24h volume of a NFT collection, on each NFT marketplace, e.g. OpenSea and Looksrare."""

        endpoint = f'nfts/{id}/tickers'

        return self.get_data(endpoint)





import requests
import logging
from coingecko.api.error import CoinGeckoAPIErrorHandler

class CoinGeckoAPI:
    __PRO_API_BASE_URL = 'https://pro-api.coingecko.com/api/v3'
    __API_BASE_URL = 'https://api.coingecko.com/api/v3'

    def __init__(self, api_key: str, pro: bool = False, session: requests.Session = None):
        if not api_key:
            raise ValueError("API key is required.")

        self.api_key = api_key
        self.base_url = self.__PRO_API_BASE_URL if pro else self.__API_BASE_URL
        self.api_key_header = 'x-cg-pro-api-key' if pro else 'x-cg-demo-api-key'
        self.session = session or requests.Session()

    def get_data(self, endpoint, params=None):
        url = f'{self.base_url}/{endpoint}'
        headers = {'Content-Type': 'application/json', f'{self.api_key_header}': self.api_key}

        response = self.session.get(url, headers=headers, params=params)
        logging.info(f"GET {url} : {response.status_code}")

        if response.status_code == 200:
            return response.json()
        else:
            CoinGeckoAPIErrorHandler.handle_response(response)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class Categories(CoinGeckoAPI):
    @process_function_args
    def get_list(self) -> list:
        """List all categories"""
        endpoint = 'coins/categories/list'

        return self.get_data(endpoint)

    @process_function_args
    def get(self, order: str = "market_cap_desc") -> list:
        """List all categories with market data"""
        endpoint = f'coins/categories'

        params = {
            'order': order
        }

        return self.get_data(endpoint, params=params)



from coingecko.api.base import CoinGeckoAPI

class Companies(CoinGeckoAPI):
    def get_public_trasuries(self, coin_id: str = "bitcoin") -> dict:
        """Get public companies bitcoin/ethereum holdings"""
        endpoint = f'companies/public_treasury/{coin_id}'

        return self.get_data(endpoint)



from coingecko.api.exchanges import Exchanges

class ProExchanges(Exchanges):
    def get_volume_chart_range(self, id: str, from_timestamp: str, to_timestamp: str) -> list:
        """📈 Get historical volume data (in BTC) of an exchange, by specifying a date range (up to 31 days per call)"""

        endpoint = f'exchanges/{id}/volume_chart/range'

        params = {
            'from': from_timestamp,
            'to': to_timestamp
        }

        return self.get_data(endpoint, params=params)



from coingecko.api import CoinGeckoClient
from coingecko.api.pro.coins import ProCoins
from coingecko.api.pro.global_data import ProGlobalData
from coingecko.api.pro.exchanges import ProExchanges
from coingecko.api.pro.nfts import ProNfts
from coingecko.api.pro.token_lists import TokenLists

class CoinGeckoProClient(CoinGeckoClient):
    def __init__(self, api_key: str):
        super().__init__(api_key=api_key, pro = True)
        self._coins = None
        self._global_data = None
        self._nfts = None
        self._exchanges = None
        self._token_lists = None


    @property
    def coins(self):
        if self._coins is None:
            self._coins = ProCoins(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._coins

    @property
    def global_data(self):
        if self._global_data is None:
            self._global_data = ProGlobalData(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._global_data

    @property
    def exchanges(self):
        if self._exchanges is None:
            self._exchanges = ProExchanges(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._exchanges

    @property
    def nfts(self):
        if self._nfts is None:
            self._nfts = ProNfts(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._nfts

    @property
    def token_lists(self):
        if self._token_lists is None:
            self._token_lists = TokenLists(api_key=self.api_key, pro=self.pro, session=self.session)
        return self._token_lists



class CoinGeckoAPIErrorHandler:
    def __init__(self, response):
        self.response = response

    @classmethod
    def handle_response(cls, response):
        if response.status_code == 400:
            raise GeckoAPIBadRequest(response.status_code, response.json())
        elif response.status_code == 401:
            raise GeckoAPIUnauthorized(response.status_code, response.json())
        elif response.status_code == 429:
            raise GeckoAPITooManyRequests(response.status_code, response.json())
        elif response.status_code == 500:
            raise GeckoAPIInternalServerError(response.status_code, response.json())
        elif response.status_code == 503:
            raise GeckoAPIServiceUnavailable(response.status_code, response.json())
        else:
            raise GeckoAPIException(response.status_code, response.json())


class GeckoAPIException(Exception):
    def __init__(self, status_code = None, error_message = None):
        self.status_code = status_code
        self.error_message = error_message

    def __str__(self):
        return f"{self.status_code}: {self.error_message}"

class GeckoAPIBadRequest(GeckoAPIException):
    def __init__(self, status_code = None, error_message = None):
        super().__init__(status_code, error_message)

class GeckoAPIUnauthorized(GeckoAPIException):
    def __init__(self, status_code = None, error_message = None):
        super().__init__(status_code, error_message)

class GeckoAPITooManyRequests(GeckoAPIException):
    def __init__(self, status_code = None, error_message = None):
        super().__init__(status_code, error_message)

class GeckoAPIInternalServerError(GeckoAPIException):
    def __init__(self, status_code = None, error_message = None):
        super().__init__(status_code, error_message)

class GeckoAPIServiceUnavailable(GeckoAPIException):
    def __init__(self, status_code = None, error_message = None):
        super().__init__(status_code, error_message)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class ExchangeRates(CoinGeckoAPI):
    def get(self) -> dict:
        """Get BTC-to-Currency exchange rates"""
        endpoint = 'exchange_rates'

        return self.get_data(endpoint)



from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args


class AssetPlatform(CoinGeckoAPI):
    def get(self, filter: str = None) -> list:
        """List all asset platforms"""
        endpoint = 'asset_platforms'

        params = {
            'filter': filter
        }

        return self.get_data(endpoint, params=params)




from coingecko.api.base import CoinGeckoAPI
from coingecko.api.utils import process_function_args

class Derivatives(CoinGeckoAPI):
    def get(self) -> list:
        """List all derivative tickers"""
        endpoint = 'derivatives'

        return self.get_data(endpoint)

    def get_exchanges(self, order: str = None, per_page: int = None, page: int = None) -> list:
        """List all derivative exchanges"""
        endpoint = 'derivatives/exchanges'

        params = {
            'order': order,
            'per_page': per_page,
            'page': page
        }

        return self.get_data(endpoint, params=params)

    def get_exchanges_id(self, id: str, include_tickers: str = None) -> dict:
        """Show derivative exchange data"""
        endpoint = f'derivatives/exchanges/{id}'

        params = {
            'include_tickers': include_tickers
        }

        return self.get_data(endpoint, params=params)

    def get_exchanges_list(self) -> list:
        """List all derivative exchanges name and identifier"""
        endpoint = 'derivatives/exchanges/list'

        return self.get_data(endpoint)



from setuptools import setup, find_packages

with open('README.md') as f:
    description = f.read()

setup(
    name='coingecko',
    version='0.11.1',
    description='Python wrapper for the CoinGecko API',
    long_description=description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    url="https://github.com/khooihzhz/coingecko-python",
    author='khooihzhz',
    author_email='khooihongzhe@gmail.com',
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    install_requires=[
        'requests',
    ],
    extras_require={
        'dev': [
            'pytest-recording',
        ]
    },
    python_requires='>=3.0',
)



from coingecko.api.base import CoinGeckoAPI

class Search(CoinGeckoAPI):
    def get(self, query: str) -> dict:
        """Search for coins, categories and markets on CoinGecko"""
        endpoint = 'search'

        params = {
            'query': query
        }

        return self.get_data(endpoint, params=params)

    def get_trending(self) -> dict:
        """Get trending search coins (Top-15) on CoinGecko in the Last 24 Hours"""
        endpoint = 'search/trending'

        return self.get_data(endpoint)



def process_function_args(func):
    def wrapper(*args, **kwargs):
        args = [process_arg(arg) for arg in args]

        for key in kwargs:
            kwargs[key] = process_arg(kwargs[key])

        return func(*args, **kwargs)

    return wrapper

def process_arg(arg):
    if isinstance(arg, list):
        arg = list_to_string(arg)
    elif isinstance(arg, bool):
        arg = bool_to_string(arg)

    return arg

def list_to_string(list):
    return ','.join(list)

def bool_to_string(bool):
    return str(bool).lower()

